import os
import plaster
import pytest

here = os.path.dirname(__file__)

test_config_relpath = 'sample_configs/test_config.ini'
test_config_path = os.path.abspath(os.path.join(here, test_config_relpath))


class Test_setup_logging(object):
    @pytest.fixture(autouse=True)
    def logging(self, fake_packages, monkeypatch):
        self.basicConfig = DummyFileConfig()
        self.fileConfig = DummyFileConfig()
        monkeypatch.setattr('logging.basicConfig', self.basicConfig)
        monkeypatch.setattr('plaster_pastedeploy.fileConfig', self.fileConfig)
        monkeypatch.chdir(here)

    def _makeOne(self, uri=None):
        if uri is None:
            uri = test_config_relpath
        return plaster.get_loader(uri)

    def test_it_no_global_conf(self):
        loader = self._makeOne()
        loader.setup_logging()
        assert self.fileConfig.called

        path, defaults = self.fileConfig.args
        assert path == test_config_relpath
        assert defaults['__file__'] == test_config_path
        assert defaults['here'] == os.path.dirname(test_config_path)

    def test_it_global_conf_empty(self):
        loader = self._makeOne()
        loader.setup_logging(defaults={})
        assert self.fileConfig.called

        path, defaults = self.fileConfig.args
        assert path == test_config_relpath
        assert defaults['__file__'] == test_config_path
        assert defaults['here'] == os.path.dirname(test_config_path)

    def test_it_global_conf_not_empty(self):
        defaults = {'key': 'val'}
        loader = self._makeOne()
        loader.setup_logging(defaults=defaults)
        assert self.fileConfig.called

        path, defaults = self.fileConfig.args
        assert path == test_config_relpath
        assert defaults['__file__'] == test_config_path
        assert defaults['here'] == os.path.dirname(test_config_path)
        assert defaults['key'] == 'val'

    def test_no_logging_section(self):
        loader = self._makeOne()
        loader.get_sections = lambda *args: []
        loader.setup_logging()
        assert self.basicConfig.called
        assert self.basicConfig.args == ()
        assert self.basicConfig.kwargs == {}

    def test_egg_uri(self):
        loader = self._makeOne('egg:FakeApp#fake')
        loader.setup_logging()
        assert self.basicConfig.called
        assert self.basicConfig.args == ()
        assert self.basicConfig.kwargs == {}


class DummyFileConfig(object):
    called = False
    args = None
    kwargs = None

    def __call__(self, *args, **kwargs):
        self.called = True
        self.args = args
        self.kwargs = kwargs



class SimpleApp(object):
    def __init__(self, global_conf, local_conf, name):
        self.global_conf = global_conf
        self.local_conf = local_conf
        self.name = name

    def __call__(self, environ, start_response):
        start_response('200 OK', [('Content-type', 'text/html')])
        return ['I am: ', name]
        
    def make_app(cls, global_conf, **conf):
        return cls(global_conf, conf, 'basic')
    make_app = classmethod(make_app)




from setuptools import setup, find_packages

setup(
    name="FakeApp",
    version="1.0",
    packages=find_packages(),
    entry_points={
        'paste.app_factory': """
        basic_app=fakeapp.apps:make_basic_app
        other=fakeapp.apps:make_basic_app2
        configed=fakeapp.configapps:SimpleApp.make_app
        """,
        'paste.filter_factory': """
        caps=fakeapp.apps:make_cap_filter
        """,
        'paste.server_factory': """
        fake=fakeapp.apps:make_fake_server
        """,
    },
)



############################################################
# Apps
############################################################


def simple_app(response, environ, start_response):
    start_response('200 OK', [('Content-type', 'text/html')])
    return ['This is ', response]


def basic_app(environ, start_response):
    return simple_app('basic app', environ, start_response)


def make_basic_app(global_conf, **conf):
    return basic_app


############################################################
# Filters
############################################################

def make_cap_filter(global_conf, method_to_call='upper'):
    def cap_filter(app):
        return CapFilter(app, global_conf, method_to_call)

    return cap_filter


class CapFilter(object):
    def __init__(self, app, global_conf, method_to_call='upper'):
        self.app = app
        self.method_to_call = method_to_call
        self.global_conf = global_conf

    def __call__(self, environ, start_response):
        app_iter = self.app(environ, start_response)
        for item in app_iter:
            yield getattr(item, self.method_to_call)()
        if hasattr(app_iter, 'close'):
            app_iter.close()


############################################################
# Servers
############################################################

def make_fake_server(global_conf=None, **settings):
    return Server(global_conf, settings)


class Server(object):
    def __init__(self, global_conf, settings):
        self.global_conf = global_conf
        self.settings = settings

    def __call__(self, app):
        return app



import os.path
import pkg_resources
import pytest
import sys

@pytest.fixture(scope='session')
def fake_packages():
    # we'd like to keep this scope more focused but it's proven really
    # difficult to fully monkeypatch pkg_resources and so for now we just
    # install the packages for the duration of the test suite
    test_dir = os.path.dirname(__file__)
    info_dir = os.path.join(test_dir, 'fake_packages', 'FakeApp')
    sys.path.insert(0, info_dir)
    pkg_resources.working_set.add_entry(info_dir)



import os
import plaster
import pytest

here = os.path.dirname(__file__)

test_settings_relpath = 'sample_configs/test_settings.ini'
test_settings_path = os.path.abspath(os.path.join(here, test_settings_relpath))


class TestSimpleUri(object):
    config_uri = test_settings_path

    @pytest.fixture(autouse=True)
    def loader(self, fake_packages):
        self.loader = plaster.get_loader(self.config_uri)

    def test_sections(self):
        result = self.loader.get_sections()
        assert set(result) == {'section1', 'section2'}

    def test_missing_section(self):
        result = self.loader.get_settings('missing', {'a': 'b'})
        assert result == {}

    def test_no_defaults_passed(self):
        result = self.loader.get_settings('section1')
        assert list(result.items()) == [
            ('a', 'a_val'),
            ('c', 'override_b'),
            ('b', 'default_b'),
        ]

        assert result.global_conf['default_a'] == 'default_a'
        assert result.global_conf['default_b'] == 'override_b'
        assert result.loader == self.loader

        with pytest.raises(Exception):
            self.loader.get_settings('section2')

    def test_defaults_passed(self):
        defaults = {'default_c': 'default_c'}
        result = self.loader.get_settings('section1', defaults=defaults)
        assert result['a'] == 'a_val'
        assert result['b'] == 'default_b'
        assert 'default_c' not in result

        assert result.global_conf['default_a'] == 'default_a'
        assert result.global_conf['default_b'] == 'override_b'
        assert result.global_conf['default_c'] == 'default_c'

        result = self.loader.get_settings('section2', defaults=defaults)
        assert result['a'] == 'a_val'
        assert result['b'] == 'b_val'
        assert result['c'] == 'default_c'

        assert result.global_conf['default_a'] == 'default_a'
        assert result.global_conf['default_b'] == 'default_b'
        assert result.global_conf['default_c'] == 'default_c'

class TestSectionedURI(TestSimpleUri):
    config_uri = test_settings_path + '#section1'

    def test_no_section_name_passed(self):
        result = self.loader.get_settings()
        assert result['a'] == 'a_val'
        assert result['b'] == 'default_b'
        assert result['c'] == 'override_b'
        assert 'default_b' not in result


class TestFullURI(TestSectionedURI):
    config_uri = 'pastedeploy+ini:' + test_settings_path + '#section1'


class TestEggURI(object):
    config_uri = 'pastedeploy+egg:FakeApp#basic_app'

    @pytest.fixture(autouse=True)
    def loader(self, fake_packages):
        self.loader = plaster.get_loader(self.config_uri)

    def test_sections(self):
        result = self.loader.get_sections()
        assert result == []

    def test_settings(self):
        result = self.loader.get_settings()
        assert result == {}

    def test_named_settings(self):
        result = self.loader.get_settings('missing')
        assert result == {}



import os
import plaster
import pytest

here = os.path.dirname(__file__)

test_config_path = 'sample_configs/test_config.ini'


class TestSimpleURI(object):
    config_uri = test_config_path

    @pytest.fixture(autouse=True)
    def loader(self, fake_packages, monkeypatch):
        monkeypatch.chdir(here)
        self.loader = plaster.get_loader(self.config_uri, protocols=['wsgi'])

    def test_get_wsgi_server_default(self):
        server = self.loader.get_wsgi_server()
        dummy_app = object()
        result = server(dummy_app)
        assert result is dummy_app
        assert server.settings['foo'] == 'main'

    def test_invalid_name(self):
        with pytest.raises(LookupError):
            self.loader.get_wsgi_server('invalid')


class TestSectionedURI(TestSimpleURI):
    config_uri = test_config_path + '#other'

    def test_get_wsgi_server_default(self):
        server = self.loader.get_wsgi_server()
        dummy_app = object()
        result = server(dummy_app)
        assert result is dummy_app
        assert server.settings['foo'] == 'other'


class TestSchemeAndSectionedURI(TestSectionedURI):
    config_uri = 'pastedeploy+ini:' + test_config_path + '#other'


class TestEggURI(object):
    config_uri = 'egg:FakeApp#fake'

    @pytest.fixture(autouse=True)
    def loader(self, fake_packages):
        self.loader = plaster.get_loader(self.config_uri, protocols=['wsgi'])

    def test_it(self):
        server = self.loader.get_wsgi_server()
        result = server('foo')
        assert result == 'foo'

    def test_it_override_name(self):
        server = self.loader.get_wsgi_server('fake')
        result = server('foo')
        assert result == 'foo'

    def test_invalid_name(self):
        with pytest.raises(LookupError):
            self.loader.get_wsgi_server('invalid')



import os
import plaster
import pytest

here = os.path.dirname(__file__)

test_filter_path = 'sample_configs/test_filter.ini'


class TestSimpleURI(object):
    config_uri = test_filter_path

    @pytest.fixture(autouse=True)
    def loader(self, fake_packages, monkeypatch):
        monkeypatch.chdir(here)
        self.loader = plaster.get_loader(self.config_uri, protocols=['wsgi'])

    def test_get_wsgi_filter(self):
        import fakeapp.apps
        app_filter_factory = self.loader.get_wsgi_filter('filt')

        other_loader = plaster.get_loader(
            'sample_configs/basic_app.ini', protocols=['wsgi'])
        app = other_loader.get_wsgi_app()
        app_filter = app_filter_factory(app)

        assert isinstance(app_filter, fakeapp.apps.CapFilter)
        assert app_filter.method_to_call == 'lower'
        assert app_filter.app is fakeapp.apps.basic_app

    def test_invalid_name(self):
        with pytest.raises(LookupError):
            self.loader.get_wsgi_filter('invalid')


class TestSectionedURI(TestSimpleURI):
    config_uri = test_filter_path + '#filt'

    def test_get_wsgi_filter(self):
        import fakeapp.apps
        app_filter_factory = self.loader.get_wsgi_filter()

        other_loader = plaster.get_loader(
            'pastedeploy+ini:sample_configs/basic_app.ini#main',
            protocols=['wsgi'])
        app = other_loader.get_wsgi_app()
        app_filter = app_filter_factory(app)

        assert isinstance(app_filter, fakeapp.apps.CapFilter)
        assert app_filter.method_to_call == 'lower'
        assert app_filter.app is fakeapp.apps.basic_app


class TestSchemeAndSectionedURI(TestSectionedURI):
    config_uri = 'pastedeploy+ini:' + test_filter_path + '#filt'


class TestEggURI(object):
    config_uri = 'egg:FakeApp#caps'

    @pytest.fixture(autouse=True)
    def loader(self, fake_packages):
        self.loader = plaster.get_loader(self.config_uri, protocols=['wsgi'])

    def test_it(self):
        import fakeapp.apps
        filter = self.loader.get_wsgi_filter()
        filtered_app = filter('foo')
        assert isinstance(filtered_app, fakeapp.apps.CapFilter)

    def test_it_override_name(self):
        import fakeapp.apps
        filter = self.loader.get_wsgi_filter('caps')
        filtered_app = filter('foo')
        assert isinstance(filtered_app, fakeapp.apps.CapFilter)

    def test_invalid_name(self):
        with pytest.raises(LookupError):
            self.loader.get_wsgi_filter('invalid')



""" Tests for plaster_pastedeploy.ConfigDict
"""
import copy                     # noqa: F401
import plaster
import pytest


@pytest.fixture
def loader():
    from plaster_pastedeploy import Loader
    uri = plaster.PlasterURL('pastedeploy+ini', 'development.ini')
    return Loader(uri)


def dict_copy(d):
    return d.copy()


def copy_copy(d):
    return copy.copy(d)


@pytest.mark.parametrize('copier',
                         [dict_copy, copy_copy],
                         ids=lambda f: f.__name__)
def test_copy(copier, loader):
    from plaster_pastedeploy import ConfigDict
    x = []
    global_conf = {}
    configdict = ConfigDict({'x': x}, global_conf, loader)

    duplicate = copier(configdict)

    assert duplicate.items() == configdict.items()
    # check that we got a shallow copy
    assert duplicate['x'] is x
    assert duplicate.global_conf is global_conf
    assert duplicate.loader is loader



import plaster

def test___repr__():
    from plaster_pastedeploy import Loader
    uri = plaster.PlasterURL('pastedeploy+ini', 'development.ini')
    loader = Loader(uri)
    assert str(loader) == (
        'plaster_pastedeploy.Loader(uri="pastedeploy+ini://development.ini")'
    )



import os
import plaster
import pytest

here = os.path.dirname(__file__)
test_config_relpath = 'sample_configs/test_config.ini'
test_config_path = os.path.abspath(os.path.join(here, test_config_relpath))


class TestFullURI(object):
    @pytest.fixture(autouse=True)
    def loader(self, fake_packages, monkeypatch):
        monkeypatch.chdir(here)
        self.loader = plaster.get_loader(
            test_config_relpath, protocols=['wsgi'])

    def test_get_wsgi_app_settings(self):
        result = self.loader.get_wsgi_app_settings('test_get')
        assert result == {'def1': 'a', 'foo': 'TEST'}
        assert result.global_conf['def1'] == 'a'
        assert result.global_conf['def2'] == 'TEST'
        assert 'basepath' in result.global_conf

    def test_invalid_name(self):
        with pytest.raises(LookupError):
            self.loader.get_wsgi_app_settings('invalid')

    def test_foreign_config(self):
        result = self.loader.get_wsgi_app_settings('test_foreign_config')
        assert result == {'another': 'FOO', 'bob': 'your uncle'}
        assert result.global_conf['def1'] == 'a'
        # NOTE this is actually different on pastedeploy tip but unreleased
        assert result.global_conf['def2'] == 'from include'
        assert result.global_conf['def3'] == 'c'
        assert result.global_conf['glob'] == 'override'
        assert 'basepath' in result.global_conf


class TestSimpleURI(object):
    @pytest.fixture(autouse=True)
    def loader(self, fake_packages, monkeypatch):
        monkeypatch.chdir(here)
        self.loader = plaster.get_loader(
            'sample_configs/test_filter_with.ini', protocols=['wsgi'])

    def test_get_wsgi_app_settings(self):
        conf = self.loader.get_wsgi_app_settings()
        assert conf['example'] == 'test'


class TestEggURI(object):
    config_uri = 'egg:FakeApp#configed'

    @pytest.fixture(autouse=True)
    def loader(self, fake_packages):
        self.loader = plaster.get_loader(self.config_uri, protocols=['wsgi'])

    def test_it(self):
        conf = self.loader.get_wsgi_app_settings()
        assert conf == {}

    def test_invalid_name(self):
        with pytest.raises(LookupError):
            self.loader.get_wsgi_app_settings('invalid')



from setuptools import setup, find_packages

def readfile(name):
    with open(name) as f:
        return f.read()

readme = readfile('README.rst')
changes = readfile('CHANGES.rst')

install_requires = [
    'PasteDeploy >= 1.5.0',  # py3 compat
    'plaster >= 0.5',  # file schemes
]

tests_require = [
    'pytest',
    'pytest-cov',
]

setup(
    name='plaster_pastedeploy',
    version='0.5',
    description=(
        'A loader implementing the PasteDeploy syntax to be used by plaster.'
    ),
    long_description=readme + '\n\n' + changes,
    author='Hunter Senft-Grupp',
    author_email='pylons-discuss@googlegroups.com',
    url='https://github.com/Pylons/plaster_pastedeploy',
    packages=find_packages('src', exclude=['tests']),
    package_dir={'': 'src'},
    include_package_data=True,
    python_requires='>=2.7,!=3.0.*,!=3.1.*,!=3.2.*,!=3.3.*',
    install_requires=install_requires,
    extras_require={
        'testing': tests_require,
    },
    zip_safe=False,
    keywords='plaster pastedeploy plaster_pastedeploy ini config egg',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
    ],
    entry_points={
        'plaster.loader_factory': [
            'file+ini=plaster_pastedeploy:Loader',
            'egg=plaster_pastedeploy:Loader',
            'pastedeploy=plaster_pastedeploy:Loader',
            'pastedeploy+ini=plaster_pastedeploy:Loader',
            'pastedeploy+egg=plaster_pastedeploy:Loader',
        ],
        'plaster.wsgi_loader_factory': [
            'file+ini=plaster_pastedeploy:Loader',
            'egg=plaster_pastedeploy:Loader',
            'pastedeploy=plaster_pastedeploy:Loader',
            'pastedeploy+ini=plaster_pastedeploy:Loader',
            'pastedeploy+egg=plaster_pastedeploy:Loader',
        ],
    },
)



import os
import plaster
import pytest

here = os.path.dirname(__file__)

basic_app_relpath = 'sample_configs/basic_app.ini'
basic_app_path = os.path.abspath(os.path.join(here, basic_app_relpath))


class TestSimpleURI(object):
    config_uri = basic_app_path

    @pytest.fixture(autouse=True)
    def loader(self, fake_packages, monkeypatch):
        monkeypatch.chdir(here)
        self.loader = plaster.get_loader(self.config_uri, protocols=['wsgi'])

    def test_get_wsgi_app_with_relative(self):
        import fakeapp.apps
        app = self.loader.get_wsgi_app()
        assert app is fakeapp.apps.basic_app

    def test_get_wsgi_app_main(self):
        import fakeapp.apps
        app = self.loader.get_wsgi_app('main')
        assert app is fakeapp.apps.basic_app

        same_app = self.loader.get_wsgi_app()
        assert same_app is fakeapp.apps.basic_app

    def test_invalid_name(self):
        with pytest.raises(LookupError):
            self.loader.get_wsgi_app('invalid')


class TestSectionedURI(TestSimpleURI):
    config_uri = basic_app_path + '#main'


class TestSchemeAndSectionedURI(TestSimpleURI):
    config_uri = 'pastedeploy+ini:' + basic_app_path + '#main'


class TestRelativeURI(TestSimpleURI):
    config_uri = basic_app_relpath


class TestRelativeSectionedURI(TestSectionedURI, TestRelativeURI):
    config_uri = basic_app_relpath + '#main'


class TestRelativeSchemeAndSectionedURI(TestSchemeAndSectionedURI,
                                        TestRelativeURI):
    config_uri = 'pastedeploy+ini:' + basic_app_relpath + '#main'


class TestEggURI(object):
    config_uri = 'egg:FakeApp#basic_app'

    @pytest.fixture(autouse=True)
    def loader(self, fake_packages):
        self.loader = plaster.get_loader(self.config_uri, protocols=['wsgi'])

    def test_it(self):
        import fakeapp.apps
        app = self.loader.get_wsgi_app()
        assert app is fakeapp.apps.basic_app

    def test_it_override_name(self):
        import fakeapp.configapps
        app = self.loader.get_wsgi_app('configed')
        assert isinstance(app, fakeapp.configapps.SimpleApp)

    def test_invalid_name(self):
        with pytest.raises(LookupError):
            self.loader.get_wsgi_app('invalid')
